/** 
 * MonitorJobServ.java
 * This servlet handles calls from EventGenerator about backend events (job 
 * submitted and job completed).  Stores this in a persistent hashtable in
 * the user's home directory.
 *
 * Possibly I need to also provide some way to notify the rest of the program
 * when the job is done.  The correct way I think would be to put a parameter
 * "Status" in the session's ContextData.  MonitorJobServ would set this to
 * "Finished" when the job was completed.   
 *
 * We have a mini-protocal for interacting with the servlet.  The first two
 * are generated on the backend.
 * + event=started: a job has been submitted to the queue.
 * + event=finished: the queue'd job is finished.
 * + event=update: append the job's id number to the hashtable.
 *
 * The next two come from the user interface:
 * + event=query: servlet has received a request to display contents of jobHash
 * + event=delete: the indicated job should be deleted from jobHash if it
 *                  has finished running. 
 * 
 * Other parameters:
 * + user: The username of the server
 * + job: The name of the job, or more precisely the problem description name.
 * + host: This is the backend machine.
 * + sleep: Tells the servlet to wait the specified number of seconds.
 *          Since java uses milliseconds internally, must multiply this
 *          by 1000.
 *
 * Logging is handled by writing to the file /tmp/monitor.log.  This file
 * is appended to by each call, so I begin each servlet call by logging the
 * time (and date).  The log is accessed by the command mylog("some string").
 * 
 * The servlet is accessed by both the EventGenerator and directly by the
 * user.  In either case, we need to know who the user is.  For notify.pl,
 * this is passed as a parameter.  For calls directly from the user, we
 * get the user name from either (1) http authorization, or (2) the server-side
 * cookie set by Charon.  You can toggle between the two by setting the
 * UseCharon boolean to true/false.
 */

package WebFlow.Servlets;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.util.*;
import java.net.*;
import java.text.*;

public class MonitorJobServ extends HttpServlet {

    //Variables 
    String userName;
    String jobName;
    String event;
    String descPath;
    String sleepTime;
    String remoteUser,remoteDir,remoteHost;
    String jobID;
    String[] filesToCopy;
    String tmp,localFile,remoteFile;  //A couple of convenient string holders.
    Hashtable jobHash=null;
    PrintWriter printOut,logOut;
    boolean UseCharon=false;  //true=ASC, false=OSC
    
    String eventInfo;

    protected void doGet(HttpServletRequest req, HttpServletResponse res) 
	throws ServletException, IOException {
	doPost(req,res);
    }

    protected void doPost(HttpServletRequest req, HttpServletResponse res) 
	throws ServletException, IOException {
	
	printOut=res.getWriter();
	//Set up logging
	if((new File("/tmp/monitor.log").exists())) {
	    logOut=
		new PrintWriter(new FileWriter("/tmp/monitor.log",true));
	}
	else {
	    logOut=
		new PrintWriter(new FileWriter("/tmp/monitor.log"));
	}
	

	mylog("====================================================");
	mylog(getTime());

	//Get the parameters
	userName=req.getParameter("user");
	mylog("user: "+userName);
	jobName=req.getParameter("job");
	mylog("job: "+jobName);
	event=(req.getParameter("event")).trim();
	mylog("event: "+event);
	remoteUser=req.getParameter("remuser");
	mylog("remuser: "+remoteUser);
	remoteDir=req.getParameter("remdir");
	mylog("remuser: "+remoteDir);
	remoteHost=req.getParameter("host");
	mylog("host: "+remoteHost);
	jobID=req.getParameter("append");
	mylog("jobID: "+jobID);
	
	filesToCopy=req.getParameterValues("copyfile");
	if(filesToCopy!=null) {
	    for(int i=0;i<filesToCopy.length;i++)
		mylog("file to copy: "+filesToCopy[i]);
	}
	sleepTime=req.getParameter("sleep");

	//Request came from an applet, so figure out who the user is.
	if(userName==null)
	    userName=getUserName(req);	    

	//Establish path to the top of the user's descriptor directory.
	//NOTE: I could also recover this from the context data.
	descPath="/a/birch.csit.fsu.edu/shared/home/cyoun/GATEWAY/apache1.3.12/WWW/GOW/WEB-INF"
 	    +"/Descriptors/users/JSPClient/"+userName+File.separator;
	
	//The following are the possible events.
	if(event.equals("started"))
	    jobStarted();
	else if(event.equals("update"))
	    jobUpdate();
	else if(event.equals("finished")) 
	    jobFinished();	
	else if(event.equals("query"))
	    showTable();
	else if(event.equals("cleanup"))
	    hashClean();
	else {
	    System.err.println("Illegal event requested");
	}
	
	printOut.flush();
	printOut.close();
    }

    /**
     * New job has been started, so append it to the jobHash.
     */
    private void jobStarted() {
	openJobHash();
	appendNewJob();
	closeJobHash();
    }
    
    /**
     * We've been notified that the job's finished, so update the
     * jobHash.  Also move output files from the backend to the
     * Descriptors directory.  I may have to put in a timer to 
     * make sure the output files have been created.
     * Change to scpFileFromBackend() if appropriate.
     * Typically, we will want the servlet to halt for a few
     * seconds before doing anything, giving the backend program
     * time to clean up and write all of its output files.
     */
    private void jobFinished() {

	int delayTime=1;
	if(sleepTime!=null){
	    delayTime=Integer.parseInt(sleepTime)*1000;
	}
	mylog("delay="+delayTime);
	delayServlet(delayTime);
	openJobHash();
	updateJobHash();
	closeJobHash();	
    }

    /**
     * Append the ID of the running job to the appropriate entry of 
     * the hashtable.
     */
    private void jobUpdate() {
	openJobHash();
	appendToJob();
	closeJobHash();
    }

    //User wants an update of his/her jobs.  Show the table.
    private void showTable() {
	openJobHash();
	printHash();
	closeJobHash();
    }

    //User has requested that a hashtable entry be deleted.
    private void hashClean() {
	openJobHash();
	deleteEntry();
	printHash();
	closeJobHash();
    }

    //Look in the user's descriptor directory, open the 
    //jobHash file there, and deserialize it.  If the file does not
    //exist, create it and serialize it.
    private void openJobHash(){
	mylog("Job Hash opened");
	boolean status=openOldFile();	
	if(!status) {
	    System.err.println("Could not create the jobHash file");
	}
	//Grok the serialized object.  
	//NOTE: Hashtables implement serializable interface.
	try{
	    ObjectInputStream in=
		new ObjectInputStream(new FileInputStream(descPath+"jobHash"));
	    jobHash=(Hashtable)in.readObject();
	    in.close();
	}
	catch(IOException e){
	    mylog(e.toString());
	    e.printStackTrace();
	}
	catch(Exception e) {
	    e.printStackTrace();
	    mylog(e.toString());
	}
    }

    //Add the newly submitted job to the list.
    private void appendNewJob() {
	jobHash.put(jobName,"running");
    }

    //Append the job id to the appropriate hashtable entry.  
    //Probably should have merged this and appendNewJob.
    //Note that I'm sticking to my convention that multivalued
    //hashes are separated by a "+" sign.
    private void appendToJob(){
	String newValue=(String)jobHash.get(jobName);
	newValue+=", "+jobID;
	jobHash.put(jobName,jobID);
    }

    //Print out the hashtable to the browser.
    private void printHashBrowser() {
	mylog("printHashBrowser entered");
	Enumeration en=jobHash.keys();
	while(en.hasMoreElements()) {
	    String name=(String)en.nextElement();
	    printOut.println(name+"  "+(String)jobHash.get(name));
	}
    }

    //Return contents of jobHash to applet.
    private void printHash() {
	mylog("printHash entered");
	Enumeration en=jobHash.keys();
	if(!en.hasMoreElements()){
	    printOut.println("No submitted jobs");
	}
	else {
	    while(en.hasMoreElements()) {
		String name=(String)en.nextElement();
		String value=(String)jobHash.get(name);
		printOut.println(name+"  "+value);
	    }
	}
    }
   
    private void deleteEntry() {
	mylog("deleteEntry entered");
	if(jobHash.containsKey(jobName)) {
	    if(((String)jobHash.get(jobName)).equals("finished")){
		Object obj=jobHash.remove(jobName);
	    }
	}
    }

    //Job has finished, so update its status.
    private void updateJobHash() {
	mylog("updateJobHash entered");
	jobHash.put(jobName,"finished");	
    }

    //We're done modifying the jobHash object, so serialize it.
    private void closeJobHash() {
	mylog("closeJobhash entered");
	try {
	    ObjectOutputStream out=
		new ObjectOutputStream(new FileOutputStream(descPath+"jobHash"));
	    out.writeObject(jobHash);
	    out.close();
	}
	catch(IOException e) {
	    e.printStackTrace();
	}
	catch(Exception e){
	    e.printStackTrace();
	}
    }
    
    /**
     * Check to see if jobHash exists.  If so, return true.
     * If jobHash file does not exist, create it, instantiate a
     * Hashtable object here, and serialize it.
     * Returns false only if the createNewFile() method fails.
     */
    private boolean openOldFile() {
	mylog("openOldFile entered");
	boolean status=true;
	File jh=new File(descPath,"jobHash");
	if(!jh.exists()) {
	    try {
		status=jh.createNewFile();
		jobHash=new Hashtable();
		closeJobHash();
	    }
	    catch(Exception e) {
		e.printStackTrace();
	    }
	}
	return status;
    }
    
    /**
     * Get the user's name, either from getRemoteUser() or from the cookie,
     * as appropriate.
     */
    private String getUserName(HttpServletRequest req){
	mylog("getUserName entered");
	String myName=null;
	if(UseCharon) {
	    String cookieName="kerID";
	    Cookie cookies[]=req.getCookies();
	    Cookie myCookie=null;
	    if(cookies!=null) {
		for(int i=0;i<cookies.length;++i){
		    if(cookies[i].getName().equals(cookieName)){
			myCookie=cookies[i];
			break;
		    }
		}
	    }
	    
	    try{
		myName=URLDecoder.decode(myCookie.getValue());
	    }
	    catch(Exception e) {
            mylog("Cookie value not available");
            e.printStackTrace();
	    }
	    return myName;
	    
	}
	else {
	    myName="gateway";  req.getRemoteUser();
	    if(myName==null) 
		myName="Invalid";
	    return myName;
	}
    }
    
    /**
     * Return the time.
     */
    private String getTime() {
	DateFormat longDate=
	    DateFormat.getDateTimeInstance(DateFormat.FULL,DateFormat.FULL);
	String time=longDate.format(new Date());
	return time;
    }


    /**
     * This causes the servlet to wait a specified time before
     * executing.
     */
    private synchronized void delayServlet(long howLong) 
	throws NumberFormatException{

	mylog("Servlet sleeping");
	try {
	    wait(howLong);
	}
	catch (Exception ex) {
	    ex.printStackTrace();
	}
	mylog("Servlet awakes");
	
    }
    
    /**
     * The following method checks the copyfile input for the following hacker
     * characters: ;,$,&,!,|,<,>,^,{,},().
     * If the input contains them, the method returns false.  Otherwise,
     * the method returns true.
     */
    private boolean checkForHack(String input) {
	String hackChar=";$&!|<>^{}()";
	for(int i=0;i<hackChar.length();i++){
	    for(int j=0;j<input.length();j++){
		if(hackChar.substring(i,i+1).equals(input.substring(j,j+1)))
		    return false;
	    }
	}
	return true;
    }
    
    /**
     * Log all relative information into the file /tmp/monitor.log
     */

    private void mylog(String logline){
	logOut.println(logline);
	logOut.flush();
    }
}

