/**
 * Program Name: parseGeneral.java
 * Comments: Provides a method for handling the JobDesc.xml.  This file
 * contains jobs and tasks.  A job is a single program to run, while a
 * task is a composition of jobs (one or more).
 *
 * Top level children of the job description can be either tasks or
 * jobs.  Likewise, a task can have children that are jobs, but a 
 * job will not have children that are tasks.  A task can have 
 * children that are themselves tasks.
 */

package extensions.generic;

import com.ibm.xml.*;
import com.ibm.xml.parser.*;
import com.ibm.xml.parsers.*;
import org.w3c.dom.*;

import java.util.*;
import org.escience.XSIL.*;
import java.io.*;

public class parseGeneral extends XSIL {

    XSIL root;
    Parser parser;
    Document doc;
    
    //These must be static for some reason.  
    static Hashtable jobHash=new Hashtable();
    static Hashtable taskHash=new Hashtable();
    Hashtable descParams=new Hashtable();

    /**
     * Empty contructor for use as true bean.  Call init() to 
     * initialize.
     */
    public parseGeneral(){
    }

    /**
     * A contructor that sets up the bean.
     * @param xmlFileName full path name of the JobDesc.xml file
     * @param url the url of the xsil dtd.
     */ 
    public void parseGeneral(String xmlFileName,String url) {
	init(xmlFileName,url);
    }

    /**
     * Initialize the object.
     * @param xmlFileName full path name of the JobDesc.xml file
     * @param url the url of the xsil dtd.
     */ 
    public boolean init(String xmlfilename, String url) {
	boolean success=false;
	//boolean ss = true;
	//root = new XSIL(xmlfilename,ss);

        if (xmlfilename == null) {
            System.err.println("Missing xmlfilename.");
	    return success;
        }
	
        try {
            // Open XML file
	    File file=new File(xmlfilename);
	    
	    //If the JobDesc.xml file does not exist, try to create a new
	    //one.
	    if(!file.exists()) {
		boolean newdoc=false;
		newdoc=createNewDoc(xmlfilename,url);
		
		//New, empty document could not be created.
		if(!newdoc) {
		    System.err.println("Could not create new job descriptor");
		    System.err.println("Bailing out");
		    return success;
		}
	    }
	    
	    boolean ss = true;
	    root = new XSIL(xmlfilename,ss);

	    //One way or another, the xml file is now ready to open.
            FileInputStream is = new FileInputStream(xmlfilename);
	    
            // Start parsing
            parser = new Parser(xmlfilename);
	    
            // Check if there are errors
            if (parser.getNumberOfErrors() > 0) {
		System.err.println("Errors > zero");
                return success;
            }
 	    // Document is well-formed
            doc = parser.readStream(is);
	    success=true;
        } 
	catch (Exception e) {
            e.printStackTrace();
        }
	return success;
    }
    
    /**
     * The following two methods are required by the XSIL parent
     * class.  We override them here.  For current purposes there is
     * no distinction between contruct() and instantiate();
     */ 
    public void construct() {
	for(int ichild=0;ichild<getChildCount(); ichild++) {
	    XSIL x=getChild(ichild);
	    if(x instanceof Param) {
		Param p=(Param)x;
		descParams.put(p.getName(),p.getText());
	    }
	    else if(x instanceof parseJob) {
		//job=(parseJob)x;
		jobHash.put(x.getName(),(parseJob)x);
	    }
	    else if(x instanceof parseTask) {
		//task=(parseTask)x;
		taskHash.put(x.getName(),(parseTask)x);
	    }
	}
    }
    
    public void instantiate() {
	construct();
    }
    
    /**
     * Accessor methods for the hashtables that the job and task
     * set up.
     */
    public Hashtable getJobHash() { return jobHash; }
    public Hashtable getTaskHash() { return taskHash; }

    public parseJob getNamedJob(String jobName) {
	return (parseJob)jobHash.get(jobName);
    }

    public parseTask getNamedTask(String taskName) {
	return (parseTask)taskHash.get(taskName);
    }

    /**
     * Accessor methods for returning the job's list of parameter
     * names and the value for a particular 
     */
    public String[] getJobParamNames(String jobName) { 
	return ((parseJob)jobHash.get(jobName)).getParamNames(); 
    }
    
    public String getJobParamValue(String jobName,String paramName) { 
	return ((parseJob)jobHash.get(jobName)).getParamValue(paramName); 
    }

    
    /**
     * This creates a new parseJob object and adds it to the the
     * hashtable list.
     */
    public void addJob(String jobName) {
	parseJob job=new parseJob();
	job.setJobName(jobName);
	jobHash.put(jobName,job);
    }
    
    /**
     * Finds the named job from the hashtable and adds the parameter
     * name and value to it.  It the named parameter already exists, 
     * the value is updated.
     */
    public void addJobParams(String jobName,
			     String paramName,
			     String paramValue) {
	parseJob job=(parseJob)jobHash.get(jobName);
	job=job.setJobParam(paramName,paramValue);
	jobHash.put(jobName,job);
    }

    /**
     * Removes the specificed job from the vector list
     */
    public void removeJob(String jobName) {
	jobHash.remove(jobName);
    }
    
    
    /**
     * Attempt to create a new JobDesc.xml document.  
     */
    private boolean createNewDoc(String xmlfilename,String url) {
	String dtdurl=url+"/xsil.dtd";

	try {
	    doc=(Document)Class.forName("com.ibm.xml.parser.TXDocument").newInstance();
	    ((TXDocument)doc).setVersion("1.0");
	    DTD dtd=((TXDocument)doc).createDTD("XSIL",
						new ExternalID(dtdurl));
	    
	    Element root=doc.createElement("XSIL");
	    Element firstchild=doc.createElement("XSIL");
	    firstchild.setAttribute("Name","Job Descriptor");
	    firstchild.setAttribute("Type","generic.parseGeneral");
	    root.appendChild(firstchild);
	    doc.appendChild(dtd);
	    doc.appendChild(root);
	    saveDocToFile(xmlfilename);
	    return true;
	}
	catch(Exception ex) {
	    ex.printStackTrace();
	    return false;
	}
    }

    /**
     * Save the document to a file.
     * @param xmlFileName the name of the file to write xml into.
     */ 
    public void saveDocToFile(String xmlFileName) {
	try {
	    PrintWriter xmlFileOut=
		new PrintWriter(new FileWriter(xmlFileName, false));
	    ((TXDocument)doc).printWithFormat(xmlFileOut);
	} 
	catch(IOException e){
	    System.err.println("Error opening xmlFileOut.");
	    System.exit(0);
	}
    }
    
    /**
     * Adds a parseJob to a parent.  Currently the  parent must be the
     * top level document. 
     */
    public void addParseJobNode(String jobName,XSIL parent) {
	parseJob job=getNamedJob(jobName);
	Element parentNode=null,grandparentNode=null;
	if(parent instanceof parseGeneral) {
	    grandparentNode=doc.getDocumentElement();
	    Node test=grandparentNode.getFirstChild();
	    while(test.getNodeType()!=Node.ELEMENT_NODE)
		test=test.getNextSibling();
	    parentNode=(Element)test;
	}
	else {
	    System.err.println("Improper attempt to add job");
	    return;
	}

	//See if the named child already exists.
	NodeList children=parentNode.getChildNodes();
	boolean hasNamedChild=false;
	Element item=null;
	for(int i=0;i<children.getLength();i++) {
	    if(children.item(i).getNodeName().equals("XSIL")) {
		Element test=(Element)children.item(i);
		if(test.getAttribute("Name").equals(jobName)) {
		    item=test;
		    hasNamedChild=true;
		    break;
		}
	    }		    

	}
	Element childitem=null	;

	item=doc.createElement("XSIL");
	item.setAttribute("Name",job.getJobName());
	item.setAttribute("Type","generic.parseJob");
	Hashtable hash=job.getJobHash();
	String[] paramNames=job.getParamNames();
	for(int i=0;i<paramNames.length;i++) {
	    childitem=doc.createElement("Param");
	    childitem.setAttribute("Name",paramNames[i]);
	    childitem.appendChild(doc.createTextNode((String)hash.get(paramNames[i])));
	    item.appendChild(childitem);
	}
	
	parentNode.appendChild(item);
    }

    /**
     * Removes the specified node from the xml tree.
     */ 
    public void removeParseJobNode(String jobName, XSIL parent) {
	Element parentNode=null, gparentNode=null;
	
	if(parent instanceof parseGeneral) {
	    gparentNode=doc.getDocumentElement();
	    Node test=gparentNode.getFirstChild();
	    while(test.getNodeType()!=Node.ELEMENT_NODE)
		test=test.getNextSibling();
	    parentNode=(Element)test;

	    NodeList children=parentNode.getChildNodes();
	    
	    for(int i=0;i<children.getLength();i++) {
		if(children.item(i).getNodeType()==Node.ELEMENT_NODE) {
		    Element child=(Element)children.item(i);
		    if(child.getAttribute("Name").equals(jobName)) {
			parentNode.removeChild(child);
			break;
		    }
		}
	    }
	}
	else {
	    System.err.println("Improper attempt to add job");
	    return;
	}	    
    }

    /**
     * Updates the xml file.  Just deletes the old node and creates
     * a new node with the new values.  Perhaps not the most efficient
     * way of doing this.
     */
    public void updateParseJobNode(String jobName,XSIL parent) {
	removeParseJobNode(jobName,parent);
	addParseJobNode(jobName,parent);
    }

    //Not yet implemented.
    public void addTask(XSIL parent) {
    }

    

}

